﻿using UnityEngine;
using System;
using System.IO;
using System.Collections;
using KiiCorp.Cloud.Storage;

[ExecuteInEditMode()]
public class Main : MonoBehaviour
{
    //    private string username = "user" + DateTime.Now.Ticks;
    //    private string password = "password";
    private string message = "";

    void OnGUI()
    {
        GUI.Label(new Rect(10, 10, 800, 50), GetType().ToString());
        if (GUI.Button(new Rect(10, 60, 250, 100), "Upload & Download"))
        {
            KiiUser.RegisterAsPseudoUser(null, (KiiUser user, Exception e1) =>
            {
                if (e1 == null)
                {
                    KiiObject obj = Kii.Bucket("images").NewKiiObject();
                    obj ["timestamp"] = DateTime.Now.Ticks;
                    obj.Save((KiiObject o1, Exception e2) =>
                    {
                        if (e2 == null)
                        {
                            byte[] image = ReadImage();
                            Stream body = new MemoryStream(image);
                            o1.UploadBody("image/png", body, (KiiObject o2, Exception e3) =>
                            {
                                if (e3 == null)
                                {
                                    Stream outputStream = new MemoryStream();
                                    o2.DownloadBody(outputStream, (KiiObject o3, Stream s, Exception e4) =>
                                    {
                                        if (e4 == null)
                                        {
                                            s.Seek(0, SeekOrigin.Begin);
                                            byte[] downloadedImage = this.ReadStream(s);
                                            if (image.Length != downloadedImage.Length)
                                            {
                                                this.message = "ERROR: expected=" + image.Length + "bytes  actual=" + downloadedImage.Length + "bytes";
                                            } else if (!this.Compare(image, downloadedImage))
                                            {
                                                this.message = "ERROR: unexpected bytes";
                                            } else
                                            {
                                                this.message = "SUCCESS!!!!!";
                                            }
                                        } else
                                        {
                                            this.ShowException("Failed to download object body", e4);
                                        }
                                    }
                                        , (KiiObject o3, long completed, long total) =>
                                    {
                                        this.message = "Downloading... " + completed + "/" + total;
                                    });
                                } else
                                {
                                    this.ShowException("Failed to upload object body", e3);
                                }
                            }
                                , (KiiObject o2, long completed, long total) =>
                            {
                                this.message = "Uploading... " + completed + "/" + total;
                            });
                        } else
                        {
                            this.ShowException("Failed to save object", e2);
                        }
                    });
                } else
                {
                    this.ShowException("Failed to register user", e1);
                }
            });
        }
        GUI.Label(new Rect(10, 175, 500, 1000), this.message);
    }

    private byte[] ReadImage()
    {
        TextAsset asset = Resources.Load<TextAsset>("kii");
        byte[] image = asset.bytes;
        if (image == null)
        {
            Debug.Log("#####Failed to read image from file.");
        
        } else
        {
            Debug.Log("#####Load image " + image.Length + "bytes");
        }
        return image;
    }

    private byte[] ReadStream(Stream stream)
    {
        MemoryStream ms = new MemoryStream();
        try
        {
            byte[] buff = new byte[65536];
            while (true)
            {
                int read = stream.Read(buff, 0, buff.Length);
                if (read > 0)
                {
                    ms.Write(buff, 0, read);
                } else
                {
                    break;
                }
            }
            return ms.ToArray();
        } finally
        {
            ms.Dispose();
        }
    }

    private bool Compare(byte[] b1, byte[] b2)
    {
        if (b1.Length != b2.Length)
        {
            return false;
        }
        for (int i = 0; i < b1.Length; i++)
        {
            if (!b1 [i].Equals(b2 [i]))
            {
                return false;
            }
        }
        return true;
    }

    private void ShowException(string msg, Exception e)
    {
        this.message = "ERROR: " + msg + "\n" + e.GetType() + "\n";
        if (e.Data != null)
        {
            this.message += "Data=" + e.Data.ToString() + "\n";
        }
        if (e.InnerException != null)
        {
            this.message += "InnerExcepton=" + e.InnerException.GetType() + "\n";
            this.message += "InnerExcepton.Message=" + e.InnerException.Message + "\n";
            this.message += "InnerExcepton.Stacktrace=" + e.InnerException.StackTrace + "\n";
        }
        this.message += "Source=" + e.Source + "\n";
        this.message += e.Message + "\n" + e.StackTrace;
    }
}
